package me.totalfreedom.discord.react;

import org.bukkit.scheduler.BukkitTask;
import reactor.core.Disposable;

/**
 * This class is a wrapper for a BukkitTask that implements the Disposable interface.
 * This is so we can schedule non-blocking tasks asynchronously on the Bukkit Scheduler.
 * <p>
 * From <a href="https://github.com/SimplexDevelopment/SimplexSS">SimplexSS</a>
 *
 * @param task The task to wrap.
 */
public record DisposableBukkitTask(BukkitTask task) implements Disposable
{
    /**
     * Disposes of the task upstream on the Bukkit scheduler.
     */
    @Override
    public void dispose()
    {
        task.cancel();
    }

    /**
     * Checks if the task is cancelled.
     *
     * @return true if the task is cancelled, false otherwise.
     */
    @Override
    public boolean isDisposed()
    {
        return task.isCancelled();
    }
}
